@@ -11,7 +11,7 @@ use std::{
 
 use crate::{
     consts::IEC,
-    core::{DevId, Device, DeviceInfo, DmName, DmOptions, DmUuid, DM},
+    core::{DevId, Device, DeviceInfo, DmFlags, DmName, DmOptions, DmUuid, DM},
     lineardev::{LinearDev, LinearDevTargetParams},
     result::{DmError, DmResult, ErrorEnum},
     shared::{
@@ -530,7 +530,7 @@ impl DmDevice<CacheDevTargetTable> for CacheDev {
     }
 
     fn teardown(&mut self, dm: &DM) -> DmResult<()> {
-        dm.device_remove(&DevId::Name(self.name()), &DmOptions::new())?;
+        dm.device_remove(&DevId::Name(self.name()), DmOptions::default())?;
         self.cache_dev.teardown(dm)?;
         self.origin_dev.teardown(dm)?;
         self.meta_dev.teardown(dm)?;
@@ -561,7 +561,7 @@ impl CacheDev {
         }
 
         let table = CacheDev::gen_default_table(&meta, &cache, &origin, cache_block_size);
-        let dev_info = device_create(dm, name, uuid, &table, &DmOptions::new())?;
+        let dev_info = device_create(dm, name, uuid, &table, DmOptions::default())?;
 
         Ok(CacheDev {
             dev_info: Box::new(dev_info),
@@ -595,7 +595,7 @@ impl CacheDev {
             device_match(dm, &dev, uuid)?;
             dev
         } else {
-            let dev_info = device_create(dm, name, uuid, &table, &DmOptions::new())?;
+            let dev_info = device_create(dm, name, uuid, &table, DmOptions::default())?;
             CacheDev {
                 dev_info: Box::new(dev_info),
                 meta_dev: meta,
@@ -619,14 +619,14 @@ impl CacheDev {
         dm: &DM,
         table: Vec<TargetLine<LinearDevTargetParams>>,
     ) -> DmResult<()> {
-        self.suspend(dm, false)?;
+        self.suspend(dm, DmOptions::default().set_flags(DmFlags::DM_NOFLUSH))?;
 
         self.origin_dev.set_table(dm, table)?;
         self.origin_dev.resume(dm)?;
 
         let mut table = self.table.clone();
         table.table.length = self.origin_dev.size();
-        self.table_load(dm, &table)?;
+        self.table_load(dm, &table, DmOptions::default())?;
 
         self.table = table;
 
@@ -644,14 +644,14 @@ impl CacheDev {
         dm: &DM,
         table: Vec<TargetLine<LinearDevTargetParams>>,
     ) -> DmResult<()> {
-        self.suspend(dm, false)?;
+        self.suspend(dm, DmOptions::default().set_flags(DmFlags::DM_NOFLUSH))?;
         self.cache_dev.set_table(dm, table)?;
         self.cache_dev.resume(dm)?;
 
         // Reload the table, even though it is unchanged. Otherwise, we
         // suffer from whacky smq bug documented in the following PR:
         // https://github.com/stratis-storage/devicemapper-rs/pull/279.
-        self.table_load(dm, self.table())?;
+        self.table_load(dm, self.table(), DmOptions::default())?;
 
         Ok(())
     }
@@ -667,14 +667,14 @@ impl CacheDev {
         dm: &DM,
         table: Vec<TargetLine<LinearDevTargetParams>>,
     ) -> DmResult<()> {
-        self.suspend(dm, false)?;
+        self.suspend(dm, DmOptions::default().set_flags(DmFlags::DM_NOFLUSH))?;
         self.meta_dev.set_table(dm, table)?;
         self.meta_dev.resume(dm)?;
 
         // Reload the table, even though it is unchanged. Otherwise, we
         // suffer from whacky smq bug documented in the following PR:
         // https://github.com/stratis-storage/devicemapper-rs/pull/279.
-        self.table_load(dm, self.table())?;
+        self.table_load(dm, self.table(), DmOptions::default())?;
 
         Ok(())
     }
@@ -727,8 +727,8 @@ impl CacheDev {
     }
 
     /// Get the current status of the cache device.
-    pub fn status(&self, dm: &DM) -> DmResult<CacheDevStatus> {
-        status!(self, dm)
+    pub fn status(&self, dm: &DM, options: DmOptions) -> DmResult<CacheDevStatus> {
+        status!(self, dm, options)
     }
 }
 
@@ -814,7 +814,7 @@ mod tests {
         let dm = DM::new().unwrap();
         let mut cache = minimal_cachedev(&dm, paths);
 
-        match cache.status(&dm).unwrap() {
+        match cache.status(&dm, DmOptions::default()).unwrap() {
             CacheDevStatus::Working(ref status) => {
                 let usage = &status.usage;
 
@@ -899,7 +899,7 @@ mod tests {
         let cache_params = LinearTargetParams::new(dev3, Sectors(0));
         let current_length = cache.meta_dev.size();
 
-        match cache.status(&dm).unwrap() {
+        match cache.status(&dm, DmOptions::default()).unwrap() {
             CacheDevStatus::Working(ref status) => {
                 let usage = &status.usage;
                 assert_eq!(*usage.total_meta * usage.meta_block_size, current_length);
@@ -916,7 +916,7 @@ mod tests {
         assert_matches!(cache.set_meta_table(&dm, table), Ok(_));
         cache.resume(&dm).unwrap();
 
-        match cache.status(&dm).unwrap() {
+        match cache.status(&dm, DmOptions::default()).unwrap() {
             CacheDevStatus::Working(ref status) => {
                 let usage = &status.usage;
                 let assigned_length = current_length + extra_length;
@@ -953,7 +953,7 @@ mod tests {
         let cache_params = LinearTargetParams::new(dev3, Sectors(0));
         let current_length = cache.cache_dev.size();
 
-        match cache.status(&dm).unwrap() {
+        match cache.status(&dm, DmOptions::default()).unwrap() {
             CacheDevStatus::Working(ref status) => {
                 let usage = &status.usage;
                 assert_eq!(*usage.total_cache * usage.cache_block_size, current_length);
@@ -970,7 +970,7 @@ mod tests {
         assert_matches!(cache.set_cache_table(&dm, cache_table.clone()), Ok(_));
         cache.resume(&dm).unwrap();
 
-        match cache.status(&dm).unwrap() {
+        match cache.status(&dm, DmOptions::default()).unwrap() {
             CacheDevStatus::Working(ref status) => {
                 let usage = &status.usage;
                 assert_eq!(
@@ -987,7 +987,7 @@ mod tests {
         assert_matches!(cache.set_cache_table(&dm, cache_table), Ok(_));
         cache.resume(&dm).unwrap();
 
-        match cache.status(&dm).unwrap() {
+        match cache.status(&dm, DmOptions::default()).unwrap() {
             CacheDevStatus::Working(ref status) => {
                 let usage = &status.usage;
                 assert_eq!(*usage.total_cache * usage.cache_block_size, current_length);
@@ -1048,8 +1048,12 @@ mod tests {
 
         let dm = DM::new().unwrap();
         let mut cache = minimal_cachedev(&dm, paths);
-        cache.suspend(&dm, false).unwrap();
-        cache.suspend(&dm, false).unwrap();
+        cache
+            .suspend(&dm, DmOptions::default().set_flags(DmFlags::DM_NOFLUSH))
+            .unwrap();
+        cache
+            .suspend(&dm, DmOptions::default().set_flags(DmFlags::DM_NOFLUSH))
+            .unwrap();
         cache.resume(&dm).unwrap();
         cache.resume(&dm).unwrap();
         cache.teardown(&dm).unwrap();
