@@ -21,7 +21,7 @@ use crate::{
         dm_flags::DmFlags,
         dm_ioctl as dmi,
         dm_options::DmOptions,
-        errors::ErrorKind,
+        errors,
         types::{DevId, DmName, DmNameBuf, DmUuid},
         util::{
             align_to, c_struct_from_slice, mut_slice_from_c_str, slice_from_c_struct,
@@ -31,20 +31,12 @@ use crate::{
     result::{DmError, DmResult, ErrorEnum},
 };
 
-/// Indicator to send IOCTL to DM
-const DM_IOCTL: u8 = 0xfd;
 #[cfg(target_os = "linux")]
 /// Control path for user space to pass IOCTL to kernel DM
 const DM_CTL_PATH: &str = "/dev/mapper/control";
 #[cfg(target_os = "android")]
 /// Control path for user space to pass IOCTL to kernel DM
 const DM_CTL_PATH: &str = "/dev/device-mapper";
-/// Major version
-const DM_VERSION_MAJOR: u32 = 4;
-/// Minor version
-const DM_VERSION_MINOR: u32 = 30;
-/// Patch level
-const DM_VERSION_PATCHLEVEL: u32 = 0;
 
 /// Start with a large buffer to make BUFFER_FULL rare. Libdm does this too.
 const MIN_BUF_SIZE: usize = 16 * 1024;
@@ -57,7 +49,7 @@ pub struct DM {
 impl DmOptions {
     /// Generate a header to be used for IOCTL.
     fn to_ioctl_hdr(
-        &self,
+        self,
         id: Option<&DevId<'_>>,
         allowable_flags: DmFlags,
     ) -> DmResult<dmi::Struct_dm_ioctl> {
@@ -65,9 +57,9 @@ impl DmOptions {
         let event_nr = u32::from(self.cookie().bits()) << 16;
         let mut hdr: dmi::Struct_dm_ioctl = Default::default();
 
-        hdr.version[0] = DM_VERSION_MAJOR;
-        hdr.version[1] = DM_VERSION_MINOR;
-        hdr.version[2] = DM_VERSION_PATCHLEVEL;
+        hdr.version[0] = dmi::DM_VERSION_MAJOR;
+        hdr.version[1] = dmi::DM_VERSION_MINOR;
+        hdr.version[2] = dmi::DM_VERSION_PATCHLEVEL;
 
         hdr.flags = clean_flags.bits();
         hdr.event_nr = event_nr;
@@ -90,17 +82,23 @@ impl DM {
     pub fn new() -> DmResult<DM> {
         Ok(DM {
             file: File::open(DM_CTL_PATH)
-                .map_err(|e| DmError::Core(ErrorKind::ContextInitError(e).into()))?,
+                .map_err(|err| DmError::Core(errors::Error::ContextInit(err.to_string())))?,
         })
     }
 
     fn hdr_set_name(hdr: &mut dmi::Struct_dm_ioctl, name: &DmName) -> DmResult<()> {
-        let _ = name.as_bytes().read(mut_slice_from_c_str(&mut hdr.name))?;
+        let _ = name
+            .as_bytes()
+            .read(mut_slice_from_c_str(&mut hdr.name))
+            .map_err(|err| errors::Error::GeneralIo(err.to_string()))?;
         Ok(())
     }
 
     fn hdr_set_uuid(hdr: &mut dmi::Struct_dm_ioctl, uuid: &DmUuid) -> DmResult<()> {
-        let _ = uuid.as_bytes().read(mut_slice_from_c_str(&mut hdr.uuid))?;
+        let _ = uuid
+            .as_bytes()
+            .read(mut_slice_from_c_str(&mut hdr.uuid))
+            .map_err(|err| errors::Error::GeneralIo(err.to_string()))?;
         Ok(())
     }
 
@@ -150,17 +148,17 @@ impl DM {
                 .as_mut()
                 .expect("pointer to own structure v can not be NULL")
         };
-        let op = request_code_readwrite!(DM_IOCTL, ioctl, size_of::<dmi::Struct_dm_ioctl>());
+        let op = request_code_readwrite!(dmi::DM_IOCTL, ioctl, size_of::<dmi::Struct_dm_ioctl>());
         loop {
             #[cfg(target_os = "android")]
             let op = op as i32;
             if let Err(err) =
                 unsafe { convert_ioctl_res!(nix_ioctl(self.file.as_raw_fd(), op, v.as_mut_ptr())) }
             {
-                let info = DeviceInfo::new(*hdr)?;
-                return Err(DmError::Core(
-                    ErrorKind::IoctlError(Box::new(info), err).into(),
-                ));
+                return Err(DmError::Core(errors::Error::Ioctl(
+                    DeviceInfo::new(*hdr).ok().map(Box::new),
+                    Box::new(err),
+                )));
             }
             // If DM was able to write the requested data into the provided buffer, break the loop
             if (hdr.flags & DmFlags::DM_BUFFER_FULL.bits()) == 0 {
@@ -174,7 +172,7 @@ impl DM {
             // the size to exceed u32::MAX.
             let len = v.len();
             if len == u32::MAX as usize {
-                return Err(DmError::Core(ErrorKind::IoctlResultTooLargeError.into()));
+                return Err(DmError::Core(errors::Error::IoctlResultTooLarge));
             }
             v.resize((len as u32).saturating_mul(2) as usize, 0);
 
@@ -198,7 +196,7 @@ impl DM {
 
     /// Devicemapper version information: Major, Minor, and patchlevel versions.
     pub fn version(&self) -> DmResult<(u32, u32, u32)> {
-        let mut hdr = DmOptions::new().to_ioctl_hdr(None, DmFlags::empty())?;
+        let mut hdr = DmOptions::default().to_ioctl_hdr(None, DmFlags::empty())?;
 
         self.do_ioctl(dmi::DM_VERSION_CMD as u8, &mut hdr, None)?;
 
@@ -212,7 +210,7 @@ impl DM {
     /// in-use devices, and they will be removed when released.
     ///
     /// Valid flags: DM_DEFERRED_REMOVE
-    pub fn remove_all(&self, options: &DmOptions) -> DmResult<()> {
+    pub fn remove_all(&self, options: DmOptions) -> DmResult<()> {
         let mut hdr = options.to_ioctl_hdr(None, DmFlags::DM_DEFERRED_REMOVE)?;
 
         self.do_ioctl(dmi::DM_REMOVE_ALL_CMD as u8, &mut hdr, None)?;
@@ -224,7 +222,7 @@ impl DM {
     /// holds their major and minor device numbers, and on kernels that
     /// support it, each device's last event_nr.
     pub fn list_devices(&self) -> DmResult<Vec<(DmNameBuf, Device, Option<u32>)>> {
-        let mut hdr = DmOptions::new().to_ioctl_hdr(None, DmFlags::empty())?;
+        let mut hdr = DmOptions::default().to_ioctl_hdr(None, DmFlags::empty())?;
         let data_out = self.do_ioctl(dmi::DM_LIST_DEVICES_CMD as u8, &mut hdr, None)?;
 
         let mut devs = Vec::new();
@@ -305,13 +303,13 @@ impl DM {
     ///
     /// // Setting a uuid is optional
     /// let name = DmName::new("example-dev").expect("is valid DM name");
-    /// let dev = dm.device_create(name, None, &DmOptions::new()).unwrap();
+    /// let dev = dm.device_create(name, None, DmOptions::default()).unwrap();
     /// ```
     pub fn device_create(
         &self,
         name: &DmName,
         uuid: Option<&DmUuid>,
-        options: &DmOptions,
+        options: DmOptions,
     ) -> DmResult<DeviceInfo> {
         let mut hdr =
             options.to_ioctl_hdr(None, DmFlags::DM_READONLY | DmFlags::DM_PERSISTENT_DEV)?;
@@ -333,7 +331,7 @@ impl DM {
     /// used.
     ///
     /// Valid flags: DM_DEFERRED_REMOVE
-    pub fn device_remove(&self, id: &DevId<'_>, options: &DmOptions) -> DmResult<DeviceInfo> {
+    pub fn device_remove(&self, id: &DevId<'_>, options: DmOptions) -> DmResult<DeviceInfo> {
         let mut hdr = options.to_ioctl_hdr(Some(id), DmFlags::DM_DEFERRED_REMOVE)?;
 
         self.do_ioctl(dmi::DM_DEV_REMOVE_CMD as u8, &mut hdr, None)?;
@@ -349,15 +347,15 @@ impl DM {
     /// Note: Possibly surprisingly, returned DeviceInfo's uuid or name field
     /// contains the previous value, not the newly set value.
     pub fn device_rename(&self, old_name: &DmName, new: &DevId<'_>) -> DmResult<DeviceInfo> {
-        let mut options = DmOptions::new();
-        let mut data_in = match *new {
-            DevId::Name(name) => name.as_bytes().to_vec(),
-            DevId::Uuid(uuid) => {
-                options.set_flags(DmFlags::DM_UUID);
-                uuid.as_bytes().to_vec()
-            }
+        let (options, id_in) = match *new {
+            DevId::Name(name) => (DmOptions::default(), name.as_bytes()),
+            DevId::Uuid(uuid) => (
+                DmOptions::default().set_flags(DmFlags::DM_UUID),
+                uuid.as_bytes(),
+            ),
         };
-        data_in.push(b'\0');
+
+        let data_in = [id_in, &[b'\0']].concat();
 
         let mut hdr = options.to_ioctl_hdr(None, DmFlags::DM_UUID)?;
         Self::hdr_set_name(&mut hdr, old_name)?;
@@ -388,9 +386,9 @@ impl DM {
     ///
     /// let name = DmName::new("example-dev").expect("is valid DM name");
     /// let id = DevId::Name(name);
-    /// dm.device_suspend(&id, &DmOptions::new().set_flags(DmFlags::DM_SUSPEND)).unwrap();
+    /// dm.device_suspend(&id, DmOptions::default().set_flags(DmFlags::DM_SUSPEND)).unwrap();
     /// ```
-    pub fn device_suspend(&self, id: &DevId<'_>, options: &DmOptions) -> DmResult<DeviceInfo> {
+    pub fn device_suspend(&self, id: &DevId<'_>, options: DmOptions) -> DmResult<DeviceInfo> {
         let mut hdr = options.to_ioctl_hdr(
             Some(id),
             DmFlags::DM_SUSPEND | DmFlags::DM_NOFLUSH | DmFlags::DM_SKIP_LOCKFS,
@@ -405,7 +403,7 @@ impl DM {
     /// methods, but if just the DeviceInfo is desired then this just
     /// gets it.
     pub fn device_info(&self, id: &DevId<'_>) -> DmResult<DeviceInfo> {
-        let mut hdr = DmOptions::new().to_ioctl_hdr(Some(id), DmFlags::empty())?;
+        let mut hdr = DmOptions::default().to_ioctl_hdr(Some(id), DmFlags::empty())?;
 
         self.do_ioctl(dmi::DM_DEV_STATUS_CMD as u8, &mut hdr, None)?;
 
@@ -423,7 +421,7 @@ impl DM {
     pub fn device_wait(
         &self,
         id: &DevId<'_>,
-        options: &DmOptions,
+        options: DmOptions,
     ) -> DmResult<(DeviceInfo, Vec<(u64, u64, String, String)>)> {
         let mut hdr = options.to_ioctl_hdr(Some(id), DmFlags::DM_QUERY_INACTIVE_TABLE)?;
 
@@ -443,10 +441,12 @@ impl DM {
     /// Documentation/device-mapper
     /// for more.
     ///
+    /// `options` Valid flags: DM_READ_ONLY, DM_SECURE_DATA
+    ///
     /// # Example
     ///
     /// ```no_run
-    /// use devicemapper::{DM, DevId, DmName};
+    /// use devicemapper::{DM, DevId, DmName, DmOptions};
     /// let dm = DM::new().unwrap();
     ///
     /// // Create a 16MiB device (32768 512-byte sectors) that maps to /dev/sdb1
@@ -460,14 +460,16 @@ impl DM {
     ///
     /// let name = DmName::new("example-dev").expect("is valid DM name");
     /// let id = DevId::Name(name);
-    /// dm.table_load(&id, &table).unwrap();
+    /// dm.table_load(&id, &table, DmOptions::default()).unwrap();
     /// ```
     pub fn table_load(
         &self,
         id: &DevId<'_>,
         targets: &[(u64, u64, String, String)],
+        options: DmOptions,
     ) -> DmResult<DeviceInfo> {
         let mut cursor = Cursor::new(Vec::new());
+
         // Construct targets first, since we need to know how many & size
         // before initializing the header.
         for (sector_start, length, target_type, params) in targets {
@@ -483,21 +485,31 @@ impl DM {
                 target_type.len() <= dst.len(),
                 "TargetType max length = targ.target_type.len()"
             );
-            let _ = target_type.as_bytes().read(dst)?;
+            let _ = target_type
+                .as_bytes()
+                .read(dst)
+                .map_err(|err| errors::Error::GeneralIo(err.to_string()))?;
 
             // Size of the largest single member of dm_target_spec
             let align_to_size = size_of::<u64>();
             let aligned_len = align_to(params.len() + 1usize, align_to_size);
             targ.next = (size_of::<dmi::Struct_dm_target_spec>() + aligned_len) as u32;
 
-            cursor.write_all(slice_from_c_struct(&targ))?;
-            cursor.write_all(params.as_bytes())?;
+            cursor
+                .write_all(slice_from_c_struct(&targ))
+                .map_err(|err| errors::Error::GeneralIo(err.to_string()))?;
+            cursor
+                .write_all(params.as_bytes())
+                .map_err(|err| errors::Error::GeneralIo(err.to_string()))?;
 
             let padding = aligned_len - params.len();
-            cursor.write_all(vec![0; padding].as_slice())?;
+            cursor
+                .write_all(vec![0; padding].as_slice())
+                .map_err(|err| errors::Error::GeneralIo(err.to_string()))?;
         }
 
-        let mut hdr = DmOptions::new().to_ioctl_hdr(Some(id), DmFlags::empty())?;
+        let mut hdr =
+            options.to_ioctl_hdr(Some(id), DmFlags::DM_READONLY | DmFlags::DM_SECURE_DATA)?;
 
         // io_ioctl() will set hdr.data_size but we must set target_count
         hdr.target_count = targets.len() as u32;
@@ -512,7 +524,7 @@ impl DM {
 
     /// Clear the "inactive" table for a device.
     pub fn table_clear(&self, id: &DevId<'_>) -> DmResult<DeviceInfo> {
-        let mut hdr = DmOptions::new().to_ioctl_hdr(Some(id), DmFlags::empty())?;
+        let mut hdr = DmOptions::default().to_ioctl_hdr(Some(id), DmFlags::empty())?;
 
         self.do_ioctl(dmi::DM_TABLE_CLEAR_CMD as u8, &mut hdr, None)?;
 
@@ -526,7 +538,7 @@ impl DM {
     /// inactive table.
     ///
     /// Valid flags: DM_QUERY_INACTIVE_TABLE
-    pub fn table_deps(&self, id: &DevId<'_>, options: &DmOptions) -> DmResult<Vec<Device>> {
+    pub fn table_deps(&self, id: &DevId<'_>, options: DmOptions) -> DmResult<Vec<Device>> {
         let mut hdr = options.to_ioctl_hdr(Some(id), DmFlags::DM_QUERY_INACTIVE_TABLE)?;
 
         let data_out = self.do_ioctl(dmi::DM_TABLE_DEPS_CMD as u8, &mut hdr, None)?;
@@ -635,14 +647,14 @@ impl DM {
     /// let name = DmName::new("example-dev").expect("is valid DM name");
     /// let id = DevId::Name(name);
     /// let res = dm.table_status(&id,
-    ///                           &DmOptions::new().set_flags(DmFlags::DM_STATUS_TABLE)).unwrap();
+    ///                           DmOptions::default().set_flags(DmFlags::DM_STATUS_TABLE)).unwrap();
     /// println!("{:?} {:?}", res.0.name(), res.1);
     /// ```
     #[allow(clippy::type_complexity)]
     pub fn table_status(
         &self,
         id: &DevId<'_>,
-        options: &DmOptions,
+        options: DmOptions,
     ) -> DmResult<(DeviceInfo, Vec<(u64, u64, String, String)>)> {
         let mut hdr = options.to_ioctl_hdr(
             Some(id),
@@ -659,7 +671,7 @@ impl DM {
     /// Returns a list of each loaded target type with its name, and
     /// version broken into major, minor, and patchlevel.
     pub fn list_versions(&self) -> DmResult<Vec<(String, u32, u32, u32)>> {
-        let mut hdr = DmOptions::new().to_ioctl_hdr(None, DmFlags::empty())?;
+        let mut hdr = DmOptions::default().to_ioctl_hdr(None, DmFlags::empty())?;
 
         let data_out = self.do_ioctl(dmi::DM_LIST_VERSIONS_CMD as u8, &mut hdr, None)?;
 
@@ -705,7 +717,7 @@ impl DM {
         sector: Option<u64>,
         msg: &str,
     ) -> DmResult<(DeviceInfo, Option<String>)> {
-        let mut hdr = DmOptions::new().to_ioctl_hdr(Some(id), DmFlags::empty())?;
+        let mut hdr = DmOptions::default().to_ioctl_hdr(Some(id), DmFlags::empty())?;
 
         let msg_struct = dmi::Struct_dm_target_msg {
             sector: sector.unwrap_or_default(),
@@ -741,8 +753,9 @@ impl DM {
     /// If DM is being used to poll for events, once it indicates readiness it
     /// will continue to do so until we rearm it, which is what this method
     /// does.
+    #[cfg(devicemapper437supported)]
     pub fn arm_poll(&self) -> DmResult<DeviceInfo> {
-        let mut hdr = DmOptions::new().to_ioctl_hdr(None, DmFlags::empty())?;
+        let mut hdr = DmOptions::default().to_ioctl_hdr(None, DmFlags::empty())?;
 
         self.do_ioctl(dmi::DM_DEV_ARM_POLL_CMD as u8, &mut hdr, None)?;
 
@@ -792,7 +805,7 @@ mod tests {
     fn sudo_test_list_devices() {
         let dm = DM::new().unwrap();
         let name = test_name("example-dev").expect("is valid DM name");
-        dm.device_create(&name, None, &DmOptions::new()).unwrap();
+        dm.device_create(&name, None, DmOptions::default()).unwrap();
 
         let devices = dm.list_test_devices().unwrap();
 
@@ -801,7 +814,7 @@ mod tests {
             vec![&*name]
         );
         assert_eq!(devices[0].2.unwrap_or(0), 0);
-        dm.device_remove(&DevId::Name(&name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&name), DmOptions::default())
             .unwrap();
     }
 
@@ -810,9 +823,9 @@ mod tests {
     fn sudo_test_create() {
         let dm = DM::new().unwrap();
         let name = test_name("example-dev").expect("is valid DM name");
-        let result = dm.device_create(&name, None, &DmOptions::new()).unwrap();
+        let result = dm.device_create(&name, None, DmOptions::default()).unwrap();
         assert_eq!(result.name(), Some(&*name));
-        dm.device_remove(&DevId::Name(&name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&name), DmOptions::default())
             .unwrap();
     }
 
@@ -823,11 +836,11 @@ mod tests {
         let name = test_name("example-dev").expect("is valid DM name");
         let uuid = test_uuid("example-363333333333333").expect("is valid DM uuid");
         let result = dm
-            .device_create(&name, Some(&uuid), &DmOptions::new())
+            .device_create(&name, Some(&uuid), DmOptions::default())
             .unwrap();
         assert_eq!(result.name(), Some(&*name));
         assert_eq!(result.uuid().unwrap(), &*uuid);
-        dm.device_remove(&DevId::Name(&name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&name), DmOptions::default())
             .unwrap();
     }
 
@@ -837,17 +850,17 @@ mod tests {
         let dm = DM::new().unwrap();
         let name = test_name("example-dev").expect("is valid DM name");
         let uuid = test_uuid("example-363333333333333").expect("is valid DM uuid");
-        dm.device_create(&name, Some(&uuid), &DmOptions::new())
+        dm.device_create(&name, Some(&uuid), DmOptions::default())
             .unwrap();
 
         let new_uuid = test_uuid("example-9999999999").expect("is valid DM uuid");
 
         assert_matches!(
             dm.device_rename(&name, &DevId::Uuid(&new_uuid)),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
 
-        dm.device_remove(&DevId::Name(&name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&name), DmOptions::default())
             .unwrap();
     }
 
@@ -857,14 +870,14 @@ mod tests {
         let dm = DM::new().unwrap();
         let name = test_name("example-dev").expect("is valid DM name");
         let uuid = test_uuid("example-363333333333333").expect("is valid DM uuid");
-        dm.device_create(&name, Some(&uuid), &DmOptions::new())
+        dm.device_create(&name, Some(&uuid), DmOptions::default())
             .unwrap();
         assert_matches!(
             dm.device_rename(&name, &DevId::Uuid(&uuid)),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
 
-        dm.device_remove(&DevId::Name(&name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&name), DmOptions::default())
             .unwrap();
     }
 
@@ -874,7 +887,7 @@ mod tests {
     fn sudo_test_set_uuid() {
         let dm = DM::new().unwrap();
         let name = test_name("example-dev").expect("is valid DM name");
-        dm.device_create(&name, None, &DmOptions::new()).unwrap();
+        dm.device_create(&name, None, DmOptions::default()).unwrap();
 
         let uuid = test_uuid("example-363333333333333").expect("is valid DM uuid");
         let result = dm.device_rename(&name, &DevId::Uuid(&uuid)).unwrap();
@@ -884,7 +897,7 @@ mod tests {
             &*uuid
         );
         assert_matches!(dm.device_info(&DevId::Uuid(&uuid)), Ok(_));
-        dm.device_remove(&DevId::Name(&name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&name), DmOptions::default())
             .unwrap();
     }
 
@@ -894,14 +907,14 @@ mod tests {
     fn sudo_test_rename_id() {
         let dm = DM::new().unwrap();
         let name = test_name("example-dev").expect("is valid DM name");
-        dm.device_create(&name, None, &DmOptions::new()).unwrap();
+        dm.device_create(&name, None, DmOptions::default()).unwrap();
 
         assert_matches!(
             dm.device_rename(&name, &DevId::Name(&name)),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
 
-        dm.device_remove(&DevId::Name(&name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&name), DmOptions::default())
             .unwrap();
     }
 
@@ -912,14 +925,14 @@ mod tests {
     fn sudo_test_rename() {
         let dm = DM::new().unwrap();
         let name = test_name("example-dev").expect("is valid DM name");
-        dm.device_create(&name, None, &DmOptions::new()).unwrap();
+        dm.device_create(&name, None, DmOptions::default()).unwrap();
 
         let new_name = test_name("example-dev-2").expect("is valid DM name");
         dm.device_rename(&name, &DevId::Name(&new_name)).unwrap();
 
         assert_matches!(
             dm.device_info(&DevId::Name(&name)),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
 
         assert_matches!(dm.device_info(&DevId::Name(&new_name)), Ok(_));
@@ -929,17 +942,17 @@ mod tests {
         assert_eq!(devices[0].0.as_ref(), &*new_name);
 
         let third_name = test_name("example-dev-3").expect("is valid DM name");
-        dm.device_create(&third_name, None, &DmOptions::new())
+        dm.device_create(&third_name, None, DmOptions::default())
             .unwrap();
 
         assert_matches!(
             dm.device_rename(&new_name, &DevId::Name(&third_name)),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
 
-        dm.device_remove(&DevId::Name(&third_name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&third_name), DmOptions::default())
             .unwrap();
-        dm.device_remove(&DevId::Name(&new_name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&new_name), DmOptions::default())
             .unwrap();
     }
 
@@ -952,7 +965,7 @@ mod tests {
                 &test_name("old_name").expect("is valid DM name"),
                 &DevId::Name(&new_name)
             ),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
     }
 
@@ -962,9 +975,9 @@ mod tests {
         assert_matches!(
             DM::new().unwrap().device_remove(
                 &DevId::Name(&test_name("junk").expect("is valid DM name")),
-                &DmOptions::new()
+                DmOptions::default()
             ),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
     }
 
@@ -973,13 +986,13 @@ mod tests {
     fn sudo_test_empty_deps() {
         let dm = DM::new().unwrap();
         let name = test_name("example-dev").expect("is valid DM name");
-        dm.device_create(&name, None, &DmOptions::new()).unwrap();
+        dm.device_create(&name, None, DmOptions::default()).unwrap();
 
         let deps = dm
-            .table_deps(&DevId::Name(&name), &DmOptions::new())
+            .table_deps(&DevId::Name(&name), DmOptions::default())
             .unwrap();
         assert!(deps.is_empty());
-        dm.device_remove(&DevId::Name(&name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&name), DmOptions::default())
             .unwrap();
     }
 
@@ -989,9 +1002,9 @@ mod tests {
         assert_matches!(
             DM::new().unwrap().table_status(
                 &DevId::Name(&test_name("junk").expect("is valid DM name")),
-                &DmOptions::new()
+                DmOptions::default()
             ),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
     }
 
@@ -1002,9 +1015,9 @@ mod tests {
         assert_matches!(
             DM::new().unwrap().table_status(
                 &DevId::Name(&name),
-                DmOptions::new().set_flags(DmFlags::DM_STATUS_TABLE)
+                DmOptions::default().set_flags(DmFlags::DM_STATUS_TABLE)
             ),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
     }
 
@@ -1017,15 +1030,15 @@ mod tests {
         let dm = DM::new().unwrap();
         let name = test_name("example-dev").expect("is valid DM name");
         let uuid = test_uuid("uuid").expect("is valid DM UUID");
-        dm.device_create(&name, Some(&uuid), &DmOptions::new())
+        dm.device_create(&name, Some(&uuid), DmOptions::default())
             .unwrap();
 
         let status = dm
-            .table_status(&DevId::Name(&name), &DmOptions::new())
+            .table_status(&DevId::Name(&name), DmOptions::default())
             .unwrap();
         assert!(status.1.is_empty());
         assert_eq!(status.0.uuid(), Some(&*uuid));
-        dm.device_remove(&DevId::Name(&name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&name), DmOptions::default())
             .unwrap();
     }
 
@@ -1036,7 +1049,7 @@ mod tests {
         let name = test_name("example_dev").expect("is valid DM name");
         assert_matches!(
             DM::new().unwrap().device_info(&DevId::Name(&name)),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
     }
 
@@ -1051,25 +1064,25 @@ mod tests {
         let name_alt = test_name("name-alt").expect("is valid DM name");
         let uuid_alt = test_uuid("uuid-alt").expect("is valid DM UUID");
 
-        dm.device_create(&name, Some(&uuid), &DmOptions::new())
+        dm.device_create(&name, Some(&uuid), DmOptions::default())
             .unwrap();
         assert_matches!(
-            dm.device_create(&name, Some(&uuid), &DmOptions::new()),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            dm.device_create(&name, Some(&uuid), DmOptions::default()),
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
         assert_matches!(
-            dm.device_create(&name, None, &DmOptions::new()),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            dm.device_create(&name, None, DmOptions::default()),
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
         assert_matches!(
-            dm.device_create(&name, Some(&uuid_alt), &DmOptions::new()),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            dm.device_create(&name, Some(&uuid_alt), DmOptions::default()),
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
         assert_matches!(
-            dm.device_create(&name_alt, Some(&uuid), &DmOptions::new()),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            dm.device_create(&name_alt, Some(&uuid), DmOptions::default()),
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
-        dm.device_remove(&DevId::Name(&name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&name), DmOptions::default())
             .unwrap();
     }
 }
