@@ -5,7 +5,7 @@
 use std::{collections::hash_set::HashSet, fmt, path::PathBuf, str::FromStr};
 
 use crate::{
-    core::{DevId, Device, DeviceInfo, DmName, DmOptions, DmUuid, DM},
+    core::{DevId, Device, DeviceInfo, DmFlags, DmName, DmOptions, DmUuid, DM},
     lineardev::{LinearDev, LinearDevTargetParams},
     result::{DmError, DmResult, ErrorEnum},
     shared::{
@@ -231,7 +231,7 @@ impl DmDevice<ThinPoolDevTargetTable> for ThinPoolDev {
     }
 
     fn teardown(&mut self, dm: &DM) -> DmResult<()> {
-        dm.device_remove(&DevId::Name(self.name()), &DmOptions::new())?;
+        dm.device_remove(&DevId::Name(self.name()), DmOptions::default())?;
         self.data_dev.teardown(dm)?;
         self.meta_dev.teardown(dm)?;
         Ok(())
@@ -445,7 +445,7 @@ impl ThinPoolDev {
         }
 
         let table = ThinPoolDev::gen_default_table(&meta, &data, data_block_size, low_water_mark);
-        let dev_info = device_create(dm, name, uuid, &table, &DmOptions::new())?;
+        let dev_info = device_create(dm, name, uuid, &table, DmOptions::default())?;
 
         Ok(ThinPoolDev {
             dev_info: Box::new(dev_info),
@@ -497,7 +497,7 @@ impl ThinPoolDev {
             device_match(dm, &dev, uuid)?;
             dev
         } else {
-            let dev_info = device_create(dm, name, uuid, &table, &DmOptions::new())?;
+            let dev_info = device_create(dm, name, uuid, &table, DmOptions::default())?;
             ThinPoolDev {
                 dev_info: Box::new(dev_info),
                 meta_dev: meta,
@@ -544,8 +544,8 @@ impl ThinPoolDev {
         let mut new_table = self.table.clone();
         new_table.table.params.low_water_mark = low_water_mark;
 
-        self.suspend(dm, false)?;
-        self.table_load(dm, &new_table)?;
+        self.suspend(dm, DmOptions::default().set_flags(DmFlags::DM_NOFLUSH))?;
+        self.table_load(dm, &new_table, DmOptions::default())?;
 
         self.table = new_table;
         Ok(())
@@ -553,8 +553,8 @@ impl ThinPoolDev {
 
     /// Get the current status of the thinpool.
     /// Returns an error if there was an error getting the status value.
-    pub fn status(&self, dm: &DM) -> DmResult<ThinPoolStatus> {
-        status!(self, dm)
+    pub fn status(&self, dm: &DM, options: DmOptions) -> DmResult<ThinPoolStatus> {
+        status!(self, dm, options)
     }
 
     /// Set the table for the existing metadata device.
@@ -568,13 +568,13 @@ impl ThinPoolDev {
         dm: &DM,
         table: Vec<TargetLine<LinearDevTargetParams>>,
     ) -> DmResult<()> {
-        self.suspend(dm, false)?;
+        self.suspend(dm, DmOptions::default().set_flags(DmFlags::DM_NOFLUSH))?;
         self.meta_dev.set_table(dm, table)?;
         self.meta_dev.resume(dm)?;
 
         // Reload the table even though it is unchanged.
         // See comment on CacheDev::set_cache_table for reason.
-        self.table_load(dm, self.table())?;
+        self.table_load(dm, self.table(), DmOptions::default())?;
 
         Ok(())
     }
@@ -590,14 +590,14 @@ impl ThinPoolDev {
         dm: &DM,
         table: Vec<TargetLine<LinearDevTargetParams>>,
     ) -> DmResult<()> {
-        self.suspend(dm, false)?;
+        self.suspend(dm, DmOptions::default().set_flags(DmFlags::DM_NOFLUSH))?;
 
         self.data_dev.set_table(dm, table)?;
         self.data_dev.resume(dm)?;
 
         let mut table = self.table.clone();
         table.table.length = self.data_dev.size();
-        self.table_load(dm, &table)?;
+        self.table_load(dm, &table, DmOptions::default())?;
 
         self.table = table;
 
@@ -681,7 +681,7 @@ mod tests {
     use std::path::Path;
 
     use crate::{
-        core::errors::{Error, ErrorKind},
+        core::{errors::Error, DmFlags},
         testing::{test_name, test_with_spec},
     };
 
@@ -695,7 +695,7 @@ mod tests {
 
         let dm = DM::new().unwrap();
         let mut tp = minimal_thinpool(&dm, paths[0]);
-        match tp.status(&dm).unwrap() {
+        match tp.status(&dm, DmOptions::default()).unwrap() {
             ThinPoolStatus::Working(ref status)
                 if status.summary == ThinPoolStatusSummary::Good =>
             {
@@ -765,11 +765,11 @@ mod tests {
                 MIN_DATA_BLOCK_SIZE / 2u64,
                 DataBlocks(1)
             ),
-            Err(DmError::Core(Error(ErrorKind::IoctlError(_, _), _)))
+            Err(DmError::Core(Error::Ioctl(_, _)))
         );
-        dm.device_remove(&DevId::Name(&meta_name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&meta_name), DmOptions::default())
             .unwrap();
-        dm.device_remove(&DevId::Name(&data_name), &DmOptions::new())
+        dm.device_remove(&DevId::Name(&data_name), DmOptions::default())
             .unwrap();
     }
 
@@ -799,7 +799,7 @@ mod tests {
         tp.set_data_table(&dm, data_table).unwrap();
         tp.resume(&dm).unwrap();
 
-        match tp.status(&dm).unwrap() {
+        match tp.status(&dm, DmOptions::default()).unwrap() {
             ThinPoolStatus::Working(ref status) => {
                 let usage = &status.usage;
                 assert_eq!(
@@ -840,7 +840,7 @@ mod tests {
         tp.set_meta_table(&dm, meta_table).unwrap();
         tp.resume(&dm).unwrap();
 
-        match tp.status(&dm).unwrap() {
+        match tp.status(&dm, DmOptions::default()).unwrap() {
             ThinPoolStatus::Working(ref status) => {
                 let usage = &status.usage;
                 assert_eq!(usage.total_meta.sectors(), 2u8 * meta_size);
@@ -863,8 +863,10 @@ mod tests {
 
         let dm = DM::new().unwrap();
         let mut tp = minimal_thinpool(&dm, paths[0]);
-        tp.suspend(&dm, false).unwrap();
-        tp.suspend(&dm, false).unwrap();
+        tp.suspend(&dm, DmOptions::default().set_flags(DmFlags::DM_NOFLUSH))
+            .unwrap();
+        tp.suspend(&dm, DmOptions::default().set_flags(DmFlags::DM_NOFLUSH))
+            .unwrap();
         tp.resume(&dm).unwrap();
         tp.resume(&dm).unwrap();
         tp.teardown(&dm).unwrap();
@@ -875,6 +877,21 @@ mod tests {
         test_with_spec(1, test_suspend);
     }
 
+    fn test_status_noflush(paths: &[&Path]) {
+        assert!(!paths.is_empty());
+
+        let dm = DM::new().unwrap();
+        let tp = minimal_thinpool(&dm, paths[0]);
+
+        tp.status(&dm, DmOptions::default().set_flags(DmFlags::DM_NOFLUSH))
+            .unwrap();
+    }
+
+    #[test]
+    fn loop_test_status_noflush() {
+        test_with_spec(1, test_status_noflush);
+    }
+
     #[test]
     fn test_thinpool_target_params_zero() {
         let result = "thin-pool 42:42 42:43 16 2 0"
